package sanketvekariya.flutter_flexible_toast;

import android.content.Context;
import android.graphics.PorterDuff;
import android.graphics.drawable.GradientDrawable;
import android.view.Gravity;
import android.view.LayoutInflater;
import android.view.View;
import android.widget.FrameLayout;
import android.widget.ImageView;
import android.widget.TextView;
import android.widget.Toast;

import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.plugin.common.PluginRegistry.Registrar;

/**
 * FlutterFlexibleToastPlugin is totally customizable toast accessible from Flutter Side Code.
 */
public class FlutterFlexibleToastPlugin implements MethodCallHandler {

    private Context mContext;
    private Toast mToast;

    private FlutterFlexibleToastPlugin(Context context) {
        mContext = context;
    }

    /**
     * Registration of Method Channel with it's name "flutter_flexible_toast"
     *
     * @param registrar
     */
    public static void registerWith(Registrar registrar) {
        final MethodChannel channel = new MethodChannel(registrar.messenger(), "flutter_flexible_toast");
        channel.setMethodCallHandler(new FlutterFlexibleToastPlugin(registrar.context()));
    }

    /**
     * Override method onMethodCall for Redesigning entire toast logic
     *
     * @param call
     * @param result
     */
    @Override
    public void onMethodCall(MethodCall call, final Result result) {

        /**
         * View Initialized for android ui code
         */
        LayoutInflater inflater = (LayoutInflater) mContext.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
        View layout = inflater.inflate(R.layout.toast_custom, null);
        TextView text = layout.findViewById(R.id.text);
        FrameLayout frameLayout = layout.findViewById(R.id.frameLayout);
        ImageView image = layout.findViewById(R.id.image);
        switch (call.method) {
            /**
             * show toast case
             */
            case "showToast":

                /**
                 * Extracted data sent through method channel from flutter code
                 */
                String mMessage = call.argument("message").toString();
                String length = call.argument("length").toString();
                String gravity = call.argument("gravity").toString();
                Number bgcolor = call.argument("bgcolor");
                Number textcolor = call.argument("textcolor");
                Number textSize = call.argument("fontSize");
                String icon = call.argument("icon");
                int mGravity;
                switch (gravity) {
                    case "top":
                        mGravity = Gravity.TOP;
                        break;
                    case "center":
                        mGravity = Gravity.CENTER;
                        break;
                    default:
                        mGravity = Gravity.BOTTOM;
                        break;
                }

                /**
                 * Toast Duration set
                 */

                int mDuration;
                if (length.equals("long")) {
                    mDuration = Toast.LENGTH_LONG;
                } else {
                    mDuration = Toast.LENGTH_SHORT;
                }

                /**
                 * Icon set as per input
                 */
                if (icon == null) {
                    image.setVisibility(View.GONE);
                } else if (icon.equals("close")) {
                    image.setVisibility(View.VISIBLE);
                    image.setImageResource(R.drawable.ic_close);
                } else if (icon.equals("error")) {
                    image.setVisibility(View.VISIBLE);
                    image.setImageResource(R.drawable.ic_error);
                } else if (icon.equals("info")) {
                    image.setVisibility(View.VISIBLE);
                    image.setImageResource(R.drawable.ic_info);
                } else if (icon.equals("success")) {
                    image.setVisibility(View.VISIBLE);
                    image.setImageResource(R.drawable.ic_success);
                } else if (icon.equals("warning")) {
                    image.setVisibility(View.VISIBLE);
                    image.setImageResource(R.drawable.ic_warning);
                } else if (icon.equals("alarm")) {
                    image.setVisibility(View.VISIBLE);
                    image.setImageResource(R.drawable.ic_alarm);
                } else if (icon.equals("location")) {
                    image.setVisibility(View.VISIBLE);
                    image.setImageResource(R.drawable.ic_location);
                } else if (icon.equals("wallet")) {
                    image.setVisibility(View.VISIBLE);
                    image.setImageResource(R.drawable.ic_wallet);
                } else if (icon.equals("dnd")) {
                    image.setVisibility(View.VISIBLE);
                    image.setImageResource(R.drawable.ic_dnd);
                }

                text.setText(mMessage);

                /**
                 * Custom style implementation with color
                 */

                GradientDrawable gradientDrawable = new GradientDrawable();
                gradientDrawable.setColor(bgcolor.intValue());
                gradientDrawable.setCornerRadius(dp2px(4.0f));
                frameLayout.setBackground(gradientDrawable);
                text.setTextSize(textSize.floatValue());
                text.setTextColor(textcolor.intValue());
                int newColor = textcolor.intValue();
                image.setColorFilter(newColor, PorterDuff.Mode.SRC_ATOP);

                mToast = new Toast(mContext);
                mToast.setDuration(mDuration);
                mToast.setView(layout);

                /**
                 * Change gravity location as per the input
                 */
                if (mGravity == Gravity.CENTER) {
                    mToast.setGravity(mGravity, 0, 0);
                } else if (mGravity == Gravity.TOP) {
                    mToast.setGravity(mGravity, 0, 100);
                } else {
                    mToast.setGravity(mGravity, 0, 100);
                }

                mToast.show();
                result.success(true);
                break;

            /**
             * cancel case
             */
            case "cancel":
                if (mToast != null) {
                    mToast.cancel();
                }
                result.success(true);
                break;

            /**
             * default case
             */
            default:
                result.notImplemented();
                break;
        }
    }

    /**
     * This method is for getting screen density of current running device
     *
     * @param dp
     * @return
     */
    private float dp2px(float dp) {
        final float scale = mContext.getResources().getDisplayMetrics().density;
        return dp * scale + 0.5f;
    }
}
